require 'gtk2'
require 'studier'
require 'rexml/document'
require 'stringio'

SAVE_FILE_NAME = 'export.xml'

class Gtk::Container
	def children_recursive()
		result = []
		each { |child|
			case child
			# Button and its subclasses
			# (radiobutton etc) inherit from
			# container...
			when Gtk::Button
				result.push(child)
			when Gtk::Container
				result += child.children_recursive
			else
				result.push(child)
			end
		}
		result
	end
end

class GStudierDialog < Gtk::Dialog
	
	# well it's a bit messy.. but little time.
	# if i'm MODIFYING an existing questionnary,
	# entries won't be nil. and there i'll find
	# the questionnary number.
	# if i'm CREATING a new questionnary, entries
	# will be nil, and questionnary_no will contain
	# the questionnary number (as Fixnum)
	def initialize(questions, entries, questionnary_no=nil)
		super()
		set_title("Questionnary values")

		@gotos = {}
		@questions = questions
		main_vbox = Gtk::VBox.new
		vbox.add(main_vbox)
		main_vbox.add(@questions_book = Gtk::Notebook.new)
# 		@questions_book.show_tabs = false
		@questions_book.scrollable = true
		@questions_book.enable_popup = true
		add_questionnary_info_tab(entries, questionnary_no)
		questions.each_with_index { |question, i|
			add_question(question, i)
		}
		button_box()
		if entries
			load_entries(entries.elements[1])
		end

		signal_connect('key-release-event') { |source, event|
			key_name = Gdk::Keyval.to_name(event.keyval)
			case key_name
			when /\d/
				manage_number_keypress(source, key_name)
			end
		}
		vbox.show_all
	end

	def manage_number_keypress(source, key_name)
		# first we find the label whose text
		# starts with "<number> - "
		children = @questions_book.get_nth_page(@questions_book.page).children_recursive
		radio = children.find { |child|
			child.kind_of?(Gtk::RadioButton) &&
				child.label =~ /^#{key_name} -/
		}
		if radio
			radio.active = true
			return
		end

		# didn't find a radio button. search more.
		label = children.find { |child|
			child.kind_of?(Gtk::Label) &&
				child.text =~ /^#{key_name} -/
		}
		
		if !label
			# nothing to do
			return
		end
		y_coord = label.translate_coordinates(source, 0, 0)[1]
		if !y_coord
			# nothing to do
			return
		end
		# now we got the Y coord of that label. find the other widget with the same Y coord.
		matching_child = children.find { |child|
			(!child.kind_of?(Gtk::Label)) &&
				(!child.kind_of?(Gtk::Entry)) &&
				(child.translate_coordinates(source, 0, 0)[1] == y_coord)
		}
		case matching_child
		when Gtk::OptionMenu
			matching_child.history = (matching_child.history + 1) % (matching_child.menu.children.size)
		end
	end

	def add_questionnary_info_tab(entries, questionnary_no)
		tab_vbox = Gtk::VBox.new
		tab_vbox.add Gtk::Label.new("Questionnary number:")
		tab_vbox.add @tb_questionnary_id = Gtk::Entry.new()
		@tb_questionnary_id.text = if entries
				entries.attributes["number"]
			else
				questionnary_no.to_s
			end
		@questions_book.append_page tab_vbox, Gtk::Label.new("Q")
	end

	def button_box()
		back_btn = Gtk::Button.new(Gtk::Stock::GO_BACK)
		back_btn.signal_connect('clicked') { |btn|
			@questions_book.prev_page
		}
		action_area.add back_btn
		fwd_btn = Gtk::Button.new(Gtk::Stock::GO_FORWARD)
		action_area.add fwd_btn
		fwd_btn.signal_connect('clicked') { |btn|
			# we need to put a -1 because the first
			# page sets the questionnary number!
			if @gotos[@questions_book.page-1]
				@questions_book.page =
					@gotos[@questions_book.page-1]
			else
				@questions_book.next_page
			end
		}
		btn_save = Gtk::Button.new(Gtk::Stock::OK)
		add_action_widget(btn_save, Gtk::Dialog::RESPONSE_OK)
	end

	def load_entries(saved_entries)
		index = 0
		@questions.each { |question|
			saved_entry = saved_entries.elements[index+1]
			raise "Question problem #{question.name} != #{saved_entry.attributes['question']}" unless question.name == saved_entry.attributes['question']
			# +1 because the first tab is the questionnary ID.
			question_vbox =
				@questions_book.get_nth_page(index+1).child.child

			case question.choices_type
			when :single_value
				load_single_value(question, question_vbox, saved_entry)
			when :choice
				load_choice_value(question, question_vbox, saved_entry)
			when :fill_all
				load_fill_all_values(question, question_vbox, saved_entry)
			end
			index += 1
		}
	end

	def utf(text)
		text.unpack("C*").pack("U*")
	end
	def from_utf(text)
		if text
			text.unpack("U*").pack("C*")
		else
			nil
		end
	end

	def question_no()
		@questions_book.get_nth_page(0).children[1].text.to_i
	end
	
	def entries_xml()
		root = REXML::Element.new "results"
		@questions.each_with_index { |question, index|
			# +1 because the first tab is the questionnary ID.
			question_vbox =
				@questions_book.get_nth_page(index+1).child.child

			elt = (case question.choices_type
			when :single_value
				 get_single_value(question, question_vbox)
			when :choice
				get_choice_value(question, question_vbox)
			when :fill_all
				get_fill_all_values(question, question_vbox)
			end)
			elt.attributes["question"] = question.name
			root.add_element elt
		}
		root
	end

	def save_entries_to_str()
		doc = REXML::Document.new
		doc.add_element entries_xml()
		str = StringIO.new
		doc.write(str, 1)
		str.string
	end

	def add_question(question, index)
		question_vbox = Gtk::VBox.new
		title_lbl = Gtk::Label.new(question.msg)
		title_lbl.wrap = true
		title_lbl.justify = Gtk::Justification::CENTER
		question_vbox.add(title_lbl)
		case question.choices_type
		when :single_value
			display_single_value(question_vbox, question)
		when :choice
			display_choice(question_vbox, question, index)
		when :fill_all
			display_fill_all(question_vbox, question)
		else
			raise "Unknown choice type: #{question.choices_type}"
		end
		scroll = Gtk::ScrolledWindow.new
		scroll.set_policy Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC
		scroll.add_with_viewport(question_vbox)
		@questions_book.append_page scroll, Gtk::Label.new(question.name)
	end

	COMBO_BOOL_NO_BO_ITEMS = %w(no yes)
	def combo_bool_no_bo()
		combo = Gtk::OptionMenu.new
		menu = Gtk::Menu.new
		COMBO_BOOL_NO_BO_ITEMS.each { |item|
			menu.add Gtk::MenuItem.new(item)
		}
		combo.menu = menu
		combo
	end
	def reverse_combo_bool_no_bo(combo)
		COMBO_BOOL_NO_BO_ITEMS[combo.history]
	end

	COMBO_BOOL_ITEMS = %w(b.o. yes no)
	def combo_bool()
		combo = Gtk::OptionMenu.new
		menu = Gtk::Menu.new
		COMBO_BOOL_ITEMS.each { |item|
			menu.add Gtk::MenuItem.new(item)
		}
		combo.menu = menu
		combo
	end

	def reverse_combo_bool(combo)
		COMBO_BOOL_ITEMS[combo.history]
	end

	def combo_order(count, question, choice_index)
		combo = Gtk::OptionMenu.new
		menu = Gtk::Menu.new
		menu.add Gtk::MenuItem.new('not selected')
		1.upto(count) { |i|
			menu.add Gtk::MenuItem.new(i.to_s)
		}
		combo.menu = menu
		combo
	end

	def display_single_value(question_vbox, question)
		case question.choices
		when :int
			radio_btn = Gtk::RadioButton.new("b.o.")
			question_vbox.add radio_btn
			hbox1 = Gtk::HBox.new
			question_vbox.add hbox1
			radio_btn1 = Gtk::RadioButton.new("value")
			hbox1.add radio_btn1
			radio_btn1.group = radio_btn
			spin = Gtk::SpinButton.new(0, 1000, 1)
			spin.sensitive = false
			hbox1.add spin
			radio_btn.signal_connect('toggled') { |btn|
				spin.sensitive = radio_btn1.active?
			}
		when :text
			scroll = Gtk::ScrolledWindow.new
			scroll.set_policy Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC
			scroll.add(Gtk::TextView.new(Gtk::TextBuffer.new))
			question_vbox.add scroll
		when :bool
			question_vbox.pack_start(combo_bool(), false, false, 25)
		else
			raise "Unknown choices: #{question.choices}"
		end
	end

	def load_single_value(question, question_vbox, saved_entry)
		raise "invalid element #{saved_entry.name}" unless saved_entry.name == "single_val"
		ctl = question_vbox.children[1]
		value = saved_entry.attributes["value"]

		case question.choices
		when :int
			# here we have in fact a hbox holding
			# the two possible radio buttons: b.o. or
			# the spinbutton.
			if value == 'b.o.'
				question_vbox.children[1].active = true
			else
				question_vbox.children[2].children[0].active = true
				question_vbox.children[2].children[1].value = value.to_i
			end
		when :text
			# child because it's a ScrolledWindow
			ctl.child.buffer.text = from_utf(value)
		when :bool
			ctl.history = COMBO_BOOL_ITEMS.index(from_utf(value))
		else
			raise "Unknown choices: #{question.choices}"
		end
	end

	def get_single_value(question, question_vbox)
		elt = REXML::Element.new "single_val"
		
		ctl = question_vbox.children[1]

		elt.attributes["value"] = 
			case question.choices
			when :int
				# here we have in fact a hbox holding
				# the two possible radio buttons: b.o. or
				# the spinbutton.
				if question_vbox.children[1].active?
					'b.o.'
				else
					question_vbox.children[2].children[1].value_as_int.to_s
				end
			when :text
				# child because it's a ScrolledWindow
				utf(ctl.child.buffer.text)
			when :bool
				utf(reverse_combo_bool(ctl))
			else
				raise "Unknown choices: #{question.choices}"
			end
		return elt
	end

	def display_choice(question_vbox, question, index)
		btns = []
		question.choices.each_with_index { |choice, i|
			radio_btn = nil
			hbox = Gtk::HBox.new
			radio_btn = Gtk::RadioButton.new(choice.description)
			hbox.add radio_btn
			if choice.wants_string
				choice_entry = Gtk::Entry.new
				choice_entry.sensitive = false
				hbox.add choice_entry
				radio_btn.signal_connect('toggled') { |btn|
					choice_entry.sensitive = radio_btn.active?
					@gotos[index] = question.gotos[i]
				}
			else
				radio_btn.signal_connect('toggled') { |btn|
					@gotos[index] = question.gotos[i]
				}
			end
			question_vbox.add hbox
			btns << radio_btn
		}
		btns[1..-1].each {|btn| btn.group = btns[0] }
		# first button is ticked by default
		@gotos[index] = question.gotos[0]
	end

	def load_choice_value(question, question_vbox, saved_entry)
		raise "invalid element #{saved_entry.name}" unless saved_entry.name == "choice_val"
		choice_index = saved_entry.attributes["index"].to_i
		hbox = question_vbox.children[choice_index+1]
		hbox.children[0].active = true
		if question.choices[choice_index].wants_string
			text = saved_entry.text
			raise "no text for choice" unless text
			hbox.children[1].text = from_utf(text)
		end
	end

	def get_choice_value(question, question_vbox)
		elt = REXML::Element.new "choice_val"

		# we don't look at the first child
		# which is the question label.
		question_vbox.children[1..-1].each_with_index { |hbox, index|
			if hbox.children[0].active?
				if question.choices[index].wants_string
					elt.text = utf(hbox.children[1].text)
				end
				
				elt.attributes["index"] = index.to_s
				return elt
			end
		}
		fail "no selected radio button for question #{question}"
	end
	
	def display_fill_all(question_vbox, question)
		# if all the choices are "values" and that
		# the values are the same for all the choices,
		# we optimize: we don't have a list of
		# checkboxes:
		# []a []b []c []d
		# []a []b []c []d
		# []a []b []c []d
		# BUT 
		# a listview:
		# a b c d
		# [][][][]
		# [][][][]
		# [][][][]
		if !question.choices.find {|choice| !choice.values}
			# all the choices are 'values'
			if question.choices.map {|ch| ch.values }.uniq.size == 1
				# only one type of 'values'
				add_listview(question_vbox, question)
				return
			end
		end
	       	
		question.choices.each_with_index { |choice, ch_index|
			vbox = Gtk::VBox.new
			hbox = Gtk::HBox.new
			vbox.add hbox
			lbl = Gtk::Label.new(choice.description)
			lbl.wrap = true
			lbl.set_alignment 0, 0
# 			hbox.add lbl
			hbox.pack_start lbl, false, false

			if choice.wants_order
				order_count = question.choices.select {|choic| choic.wants_order}.size
				if choice.order_max
					order_count = question.choices[0].order_max
				end
				hbox.add(my_combo = combo_order(order_count, question, ch_index))
			end
			if choice.wants_int
				hbox.add(Gtk::SpinButton.new(0, 1000, 1))
			end
			if choice.wants_string
				choice_entry = Gtk::Entry.new
				hbox.add choice_entry
			end
			if choice.wants_bool
				hbox.pack_end combo_bool(), false, false
			end
			if choice.wants_bool_no_bo
				hbox.pack_end combo_bool_no_bo(), false, false
			end
			if choice.wants_text
				scroll = Gtk::ScrolledWindow.new
				scroll.set_policy Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC
				scroll.add(Gtk::TextView.new(Gtk::TextBuffer.new))
				hbox.add scroll
			end
			if choice.alts
				combo = Gtk::OptionMenu.new
				menu = Gtk::Menu.new
				choice.alts.each { |alt|
					menu.add Gtk::MenuItem.new(alt)
				}
				combo.menu = menu
				hbox.pack_end combo, false, false
			end
			if choice.values
				cb_line = Gtk::HBox.new
				vbox.add cb_line
				choice.values.each { |value|
					cb_line.add Gtk::CheckButton.new(value)
				}
			end
# 			question_vbox.pack_start(vbox, false, false)
			question_vbox.pack_start(vbox, false, false, 2)
		}
	end

	def add_listview(question_vbox, question)
		choices_count = question.choices[0].values.size
		tvStore = Gtk::ListStore.new(String, *([TrueClass] * choices_count))
		tv = Gtk::TreeView.new(tvStore)
		question_vbox.add(tv)
		tv.append_column(desc_col = Gtk::TreeViewColumn.new("Question", Gtk::CellRendererText.new, 'text' => 0))
		desc_col.resizable = true
		desc_col.sizing = Gtk::TreeViewColumn::FIXED
		desc_col.fixed_width = 160
		1.upto(choices_count) { |i|
			add_bool_col(tv, tvStore, question.choices[0].values[i-1], i)

		}
		question.choices.each_with_index { |choice, ch_index|
			iter = tvStore.append
			iter[0] = choice.description
			choice.values.each_with_index { |v, i|
# 				iter[i+1] = v
				iter[i+1] = false
			}
		}
	end

	def add_bool_col(tv, tvStore, name, col)
		renderer = Gtk::CellRendererToggle.new
		renderer.signal_connect('toggled') do |cell, path|
			fixed_toggled(tvStore, path, col)
		end
		col_obj = Gtk::TreeViewColumn.new(name, renderer, 'active' => col)
		tv.append_column(col_obj)
		col_obj.resizable = true
	end

	def fixed_toggled(model, path_str, col)
		path = Gtk::TreePath.new(path_str)
		
		# get toggled iter
		iter = model.get_iter(path)
		fixed =iter[col]
		
		# do something with the value
		fixed ^= 1
		
		# set new value
		iter[col] = fixed
	end

	def load_fill_all_values(question, question_vbox, selected_value0)
		raise "invalid element #{selected_value0.name}" unless selected_value0.name == "fill_all_values"

		elements = selected_value0.elements.to_a

		if question_vbox.children[1].kind_of? Gtk::TreeView
			load_tv(question_vbox.children[1], elements)
			return
		end

		question.choices.each_with_index { |choice, ch_index|

			# +1 because the first child is the
			# description label of the question
			vbox = question_vbox.children[ch_index+1]
			hbox = vbox.children[0]
 			selected_value = elements.shift
			raise "invalid element #{selected_value.name}" unless selected_value.name == "value"
			
			# child[0] is the description label
			cur_index = 1
			if choice.wants_order
				hbox.children[cur_index].history = selected_value.attributes["order"].to_i
				cur_index += 1
			end
			if choice.wants_int
				hbox.children[cur_index].value_as_int = selected_value.attributes["int"].to_i
				cur_index += 1
			end
			if choice.wants_string
				hbox.children[cur_index].text = from_utf(selected_value.attributes["string"])
				cur_index += 1
			end
			if choice.wants_bool
				hbox.children[cur_index].history = COMBO_BOOL_ITEMS.index(from_utf(selected_value.attributes["bool"]))
				cur_index += 1
			end
			if choice.wants_bool_no_bo
				hbox.children[cur_index].history = COMBO_BOOL_NO_BO_ITEMS.index(from_utf(selected_value.attributes["bool_no_bo"]))
				cur_index += 1
			end
			if choice.wants_text
				# child because there's a ScrolledWindow.
				hbox.children[cur_index].child.buffer.text = from_utf(selected_value.attributes["text"])
				cur_index += 1
			end
			if choice.alts
				# combo box with all the choices -store index
				hbox.children[cur_index].history = selected_value.attributes["alts"].to_i
				cur_index += 1
			end
			if choice.values
				raise "invalid node #{selected_value.name}" unless selected_value.name == "value"
				_values = selected_value.elements.to_a('*/subval').map {|elt| elt.text.to_i}
				choices_hbox = vbox.children[1]
				choices_hbox.children.each_with_index { |cbutton, cb_index|
					raise "invalid type: #{cbutton}" unless cbutton.kind_of? Gtk::CheckButton
					cbutton.active = (_values.include?(cb_index))
				}
			end
		}
	end

	def load_tv(treeview, elements)
		tv_model = treeview.model
		row_index = 0
		tv_model.each { |model, path, treeiter|
			raise "invalid type: #{treeiter}" unless treeiter.kind_of? Gtk::TreeIter
# 			cbutton.active = (_values.include?(cb_index))
			element = elements[row_index]
			_values = element.elements.to_a('*/subval').map {|elt| elt.text.to_i}
			1.upto(tv_model.n_columns-1) { |i|
				treeiter[i] = _values.include?(i-1)
			}
			row_index += 1
		}
	end

	def get_fill_all_values(question, question_vbox)
		result = []
		result_node = REXML::Element.new "fill_all_values"

		if question_vbox.children[1].kind_of? Gtk::TreeView
			return get_fill_all_treeview(result_node, question_vbox.children[1])
		end
		
		question.choices.each_with_index { |choice, ch_index|
			cur_row = REXML::Element.new "value"
			
			# +1 because the first child is the
			# description label of the question
			vbox = question_vbox.children[ch_index+1]
			hbox = vbox.children[0]
			
			# child[0] is the description label
			cur_index = 1
			if choice.wants_order
				order = hbox.children[cur_index].history
				cur_index += 1
				cur_row.attributes["order"] = order.to_s
			end
			if choice.wants_int
				cur_row.attributes["int"] = hbox.children[cur_index].value_as_int.to_s
				cur_index += 1
			end
			if choice.wants_string
				cur_row.attributes["string"] = utf(hbox.children[cur_index].text)
				cur_index += 1
			end
			if choice.wants_bool
				cur_row.attributes["bool"] = utf(reverse_combo_bool(hbox.children[cur_index]))
				cur_index += 1
			end
			if choice.wants_bool_no_bo
				cur_row.attributes["bool_no_bo"] = utf(reverse_combo_bool_no_bo(hbox.children[cur_index]))
				cur_index += 1
			end
			if choice.wants_text
				# child because there's a ScrolledWindow.
				cur_row.attributes["text"] = utf(hbox.children[cur_index].child.buffer.text)
				cur_index += 1
			end
			if choice.alts
				# combo box with all the choices -store index
				cur_row.attributes["alts"] = hbox.children[cur_index].history.to_s
				cur_index += 1
			end
			if choice.values
				# here we return the array of indexes
				# of selected values
				values_tmp_line_res = []
				choices_hbox = vbox.children[1]
				vals = REXML::Element.new "values"
				choices_hbox.children.each_with_index { |cbutton, cb_index|
					raise "invalid type: #{cbutton}" unless cbutton.kind_of? Gtk::CheckButton
					if cbutton.active?
						subval = REXML::Element.new "subval"
						subval.text = utf(cb_index.to_s)
						vals.add subval
					end
				}
				cur_row.add vals
			end
			result_node.add cur_row
		}
		result_node
	end

	def get_fill_all_treeview(result_node, treeview)
		tv_model = treeview.model
		tv_model.each { |model, path, treeiter|
			val = REXML::Element.new "value"
			val.add(vals = REXML::Element.new("values"))
			1.upto(tv_model.n_columns-1) { |i|
				if treeiter[i]
					subval = REXML::Element.new "subval"
					subval.text = utf((i-1).to_s)
					vals.add subval
				end
			}
			result_node.add val
		}
		result_node
	end
end

