require 'plugins/export/shared/exportbase.rb'

class Calctables < ExportBase
	def export_desc()
		"calculation tables"
	end

	# true if i support exporting only few questionnaries,
	# exporting only the selection.
	def support_export_selection()
		true
	end

	def export(_xml_tree, tv, questions, file, export_selected_if_possible)
		if export_selected_if_possible
			xml_tree = filter_xml_tree(_xml_tree, tv)
		else
			xml_tree = _xml_tree
		end
		export_value_table(xml_tree, questions, file, '1',
			%w(1 2 3+))
		export_value_table(xml_tree, questions, file, '2',
			%w(0 1 2 3+))
		export_q4(xml_tree, questions, file)
		export_choice_several_q_choices(xml_tree, questions, file, '5')
		export_choice_q(xml_tree, questions, file, '6')
		export_choice_q(xml_tree, questions, file, '7')
		export_choice_several_q(xml_tree, questions, file, '8')
		export_q9(xml_tree, questions, file)
		export_choice_q(xml_tree, questions, file, '10')
		
		# must filter the total by which we divide for %tages:
		# only for those who entered YES in question 10.
		export_choice_several_q(xml_tree, questions, file, '10a')
		export_choice_several_q(xml_tree, questions, file, '10b')
		
		export_choice_q(xml_tree, questions, file, '11')
		export_choice_q(xml_tree, questions, file, '12')
		
		# only consider questions for which user
		# answered yes to question 12
		export_choice_q(xml_tree, questions, file, '12a', 
			filter_answer(xml_tree, '12', 1))
		
		export_choice_q(xml_tree, questions, file, '13')
		export_choice_q(xml_tree, questions, file, '14')
		export_choice_several_q_choices(xml_tree, questions, file, '15-1')
		export_choice_several_q_several_choices(xml_tree, questions, file, '15-2')
		
		export_choice_several_q(xml_tree, questions, file, '16')
		export_choice_q(xml_tree, questions, file, '17')
		export_choice_several_q(xml_tree, questions, file, '18')
		export_choice_several_q_choices(xml_tree, questions, file, '19')
		export_choice_q(xml_tree, questions, file, '21')

		# only consider questions for which user
		# answered ne to question 21
		export_choice_q(xml_tree, questions, file, '21b', 
			filter_answer(xml_tree, '21', 2))
		
		export_choice_several_q_choices(xml_tree, questions, file, '22')
		export_choice_several_q(xml_tree, questions, file, '23')
		export_order(xml_tree, questions, file, '24')
		export_choice_q(xml_tree, questions, file, '25')
		export_choice_q(xml_tree, questions, file, '26')

		# need to filter only those that were properly
		# filled in (1,2,3,4,5 are chosen)
		export_order(xml_tree, questions, file, '27', true)
		export_order_choice_count(xml_tree, questions, file, '27')

		export_choice_q(xml_tree, questions, file, '28')

		# [0..25, 26..30, 31..35, 36..40, ..., 81..85]
		groups = [0..25] + (0..11).map {|a| (26+5*a..30+5*a) }
		export_q_value_ranges(xml_tree, questions, file, "29", groups)

		export_choice_q(xml_tree, questions, file, '30')
		export_choice_q(xml_tree, questions, file, '31')
		export_choice_q(xml_tree, questions, file, '32')
 
		# must divide not by total no of questionnaries,
		# but only by those who answered yes to Q32.
		export_q32a(xml_tree, questions, file)
		
		# only for those who have children.
		export_value_table(xml_tree, questions, file, '32b',
			%w(0 1+), filter_answer(xml_tree, '32', 1))
		
		export_choice_q(xml_tree, questions, file, '33')
		export_choice_q(xml_tree, questions, file, '35')
		export_choice_q(xml_tree, questions, file, '36')
		export_choice_q(xml_tree, questions, file, '37')
		export_choice_q(xml_tree, questions, file, '38')
 
		# [60..64, 65..69, 70..74,...]
		groups = (0..7).map {|a| (60+5*a..64+5*a) } 
		export_q_value_ranges(xml_tree, questions, file, "39", groups)
		
		export_choice_q(xml_tree, questions, file, '40')
		export_choice_q(xml_tree, questions, file, '41')
		export_choice_q(xml_tree, questions, file, '42')
		export_choice_q(xml_tree, questions, file, '43')
		export_value_table(xml_tree, questions, file, '44',
			%w(1 2 3 4 5 6+))
		export_choice_q(xml_tree, questions, file, '45')
	end

	def export_q4(xml_tree, questions, file)
		file.puts "4. #{questions[3].msg}"
		file.puts "	N	%	Moški	Ženska"
		answers = get_answer_nodes(xml_tree, "4")
		choices = questions[3].choices
		choices_desc = choices.map {|ch| ch.description}
		
		total = answers.size

		gender = get_answer_nodes(xml_tree, "28").map { |g_node|
			g_node.attributes["index"].to_i
		}

		indexes = answers.map { |ans| ans.attributes["index"].to_i }
		choice_counts = (0..7).map { |i|
			indexes.select {|ind| i == ind}.size
		}

		moski_counts  = (0..7).map { |i|
			indexes.zip(gender).select { |ind, gender_v|
				(i == ind) && (gender_v == 0)
			}.size
		}
		zenska_counts = (0..7).map { |i|
			indexes.zip(gender).select { |ind, gender_v|
				(i == ind) && (gender_v == 1)
			}.size
		}

		
		0.upto(7) { |index|
			file.puts "#{choices_desc[index]}	#{choice_counts[index]}	#{percent(choice_counts[index],total)}	#{moski_counts[index]}	#{zenska_counts[index]}"
		}
		file.puts "Skupaj	#{total}"
		file.puts "\n"
	end

	def export_q9(xml_tree, questions, file)
		file.puts "9. #{questions[8].msg}"
		file.puts "	N	skupaj ur	ur/osebo"

		elt_numbers = get_single_values_int(xml_tree, "9")
		total = elt_numbers.size
		count_ur = elt_numbers.select {|a| !bo?(a)}.size
		count_bo = elt_numbers.select {|a| bo?(a)}.size
		total_ur = elt_numbers.inject(0) { |total, val|
			total + val.to_i unless bo?(val)
		}
		file.puts "odgovorilo	#{count_ur}	#{total_ur}	#{total_ur.to_f/count_ur}"
		file.puts "b.o.	#{count_bo}"
		file.puts "Skupaj	#{elt_numbers.size}"
		file.puts "\n"
	end

	def export_q32a(xml_tree, questions, file)
		question = questions.find { |q| q.name == '32a'}
		file.puts "32a. #{question.msg}"

		elt_numbers = get_single_values_int(xml_tree, '32a',
			filter_answer(xml_tree, '32', 1))
		count_children = elt_numbers.inject(0) { |so_far, elt|
			if !bo?(elt)
				so_far + elt.to_i
			else
				so_far
			end
		}

		file.puts "Skupno št. otrok	No children for those who have children	No. children for all oskrb."
		file.puts "#{count_children}	#{count_children.to_f/elt_numbers.size}	#{count_children.to_f/count_questionnaries(xml_tree)}"
		file.puts "\n"
	end
	
	def count_questionnaries(xml_tree)
		xml_tree.root.elements.size
	end
	
	# export a question with integers as values, presenting
	# the answers by groups
	# eg 0-25 years, 26-30 years etc.
	# groups == [0..25, 26..30, ..]
	def export_q_value_ranges(xml_tree, questions, file,
			question_no, groups)
		question = questions.find { |q| q.name == question_no}
		file.puts "#{question_no}. #{question.msg}"
		file.puts "	N	%"

		elt_numbers = get_single_values_int(xml_tree, question_no)
		total = elt_numbers.size
		groups_count = groups.map { |group|
			elt_numbers.select {|val| !bo?(val) && group === val.to_i }.size
		}
		bo_count = elt_numbers.select {|val| bo?(val)}.size
		desc_grp = lambda { |grp|
			if grp.begin == 0
				"do #{grp.end}"
			else
				"#{grp.begin}-#{grp.end}"
			end
		}
		groups.zip(groups_count).each { |group, group_count|
			file.puts "#{desc_grp[group]}	#{group_count}	#{percent(group_count, total)}"
		}
		file.puts "b.o.	#{bo_count}	#{percent(bo_count, total)}"
		file.puts "Skupaj	#{total}"
		file.puts "\n"
	end

	# returns a filter that
	# only considers questions for which user
	# answered ANSWER_INDEX to question QUESTION_INDEX
	def filter_answer(xml_tree, question_index, answer_index)
		lambda { |answers|
			answers_q = get_answer_nodes(xml_tree, question_index)
			index = -1
			answers.select { |answer|
				index += 1
				answers_q[index].attributes["index"].to_i == answer_index
			}
		}
	end

	def export_choice_q(xml_tree, questions, file, 
			question_no, answers_filter=nil)
		question = questions.find { |q| q.name == question_no}
		file.puts "#{question_no}. #{question.msg}"
		file.puts "	N	%"
		answers = get_answer_nodes(xml_tree, question_no)
		if answers_filter
			answers = answers_filter.call(answers)
		end
		choices = question.choices
		choices_desc = choices.map {|ch| ch.description}
		
		total = answers.size

		indexes = answers.map { |ans| ans.attributes["index"].to_i }
		choice_counts = (0..choices.size-1).map { |i|
			indexes.select {|ind| i == ind}.size
		}

		0.upto(choices.size-1) { |index|
			file.puts "#{choices_desc[index]}	#{choice_counts[index]}	#{percent(choice_counts[index],total)}"
		}
		file.puts "Skupaj	#{total}"
		file.puts "\n"
	end

	def export_choice_several_q_choices(xml_tree, questions, file,
			question_no)
		question = questions.find { |q| q.name == question_no}
		file.puts "#{question_no}. #{question.msg}"
		
		alts = question.choices[0].alts
		question_key = 'alts'
		if question.choices[0].wants_bool
			alts = %w(yes no b.o.)
			question_key = 'bool'
		end

		answers = get_answer_nodes(xml_tree, question_no)
		if answers.size == 0
			file.write "No results for question #{question_no}.\n"
			return
		end
		choices = question.choices
		choices_desc = choices.map {|ch| ch.description}

		choices_count = answers[0].elements.size

		# go through all the possible choices
		# (partner, odrasel otrok, ...) 
		counts = (0..choices_count-1).map { |cur_choice|
			a_counts = alts.map {0}
			answers.each { |answer|
				choice = answer.elements[cur_choice+1]
				raise "can't parse xml #{val.name}" unless choice.name == "value"
				
				# case for 'alts'
				answer_index = choice.attributes[question_key].to_i
				if question_key == 'bool'
					answer_index = alts.index(choice.attributes[question_key])
				end
				a_counts[answer_index] += 1
			}
			a_counts
		}
		disp_alts = alts - ['b.o.'] # display b.o. differently
		file.puts "	#{disp_alts.join("\t")}			#{disp_alts.join("\t")}\tb.o."
		# 0..-2 to remove the trailing \t
		col_titles = disp_alts.map { "N\t" }.join[0..-2]
		# 0..-2 to remove the trailing \t
		pcent_titles = disp_alts.map { "v%\t" }.join[0..-2]
		file.puts "	#{col_titles}	b.o.	Skupaj	#{pcent_titles}\tv%"
		counts.each_with_index { |count, i|
			total_row = counts[i].inject(0) {|so_far, cur| so_far + cur }
			vals_str = ''
			pcts_str = ''
			alts.each_with_index { |alt, ind|
				if alt != 'b.o.'
					vals_str += "#{counts[i][ind]}\t"
					pcts_str += "#{percent(counts[i][ind], total_row)}\t"
				end
			}
			alts.each_with_index { |alt, ind|
				if alt == 'b.o.'
					vals_str += "#{counts[i][ind]}\t"
					pcts_str += "#{percent(counts[i][ind], total_row)}\t"
				end
			}
			# [0..-2] to remove the final tab.
			file.puts "#{choices_desc[i]}	#{vals_str}#{total_row}	#{pcts_str[0..-2]}"
		}
		file.puts "\n"
	end
	
	def export_choice_several_q_several_choices(xml_tree,
		questions, file, question_no)
		question = questions.find { |q| q.name == question_no}
		file.puts "#{question_no}. #{question.msg}"
		
		alts = question.choices[0].values

		answers = get_answer_nodes(xml_tree, question_no)
		if answers.size == 0
			file.write "No results for question #{question_no}.\n"
			return
		end
		choices = question.choices
		choices_desc = choices.map {|ch| ch.description}

		choices_count = answers[0].elements.size

		# go through all the possible choices
		# (partner, odrasel otrok, ...) 
		counts = (0..choices_count-1).map { |cur_choice|
			a_counts = alts.map {0}
			answers.each { |answer|
				choice = answer.elements[cur_choice+1]
				raise "can't parse xml #{val.name}" unless choice.name == "value"

				values = choice.elements[1]
				raise "invalid xml #{values.name}" unless values.name == "values"
				values.each_element { |subval|
					answer_index = subval.text.to_i
					a_counts[answer_index] += 1
				}
			}
			a_counts
		}

		file.puts("\t" + alts.join("\t") + "\tSkupaj\t" + alts.join("\t"))
		file.puts("\t" + (alts * 2).map { |choice|
			  "N\t"
		}.join + "N")

		counts.each_with_index { |count, i|
			# part with the numbers
			line_total =  counts[i].inject(0) {|so_far, count| so_far + count }
			rest_line = (0..counts[i].size-1).map { |j|
				counts[i][j]
			}.join("\t") + "\t#{line_total}"
			# part with the %tages
			rest_line += "\t" + (0..counts[i].size-1).map { |j|
				percent(counts[i][j], line_total)
			}.join("\t")
			file.puts "#{choices_desc[i]}\t#{rest_line}"
		}

		file.puts "\n"
	end

	def export_choice_several_q(xml_tree, questions, file,
			question_no)
		question = questions.find { |q| q.name == question_no}
		file.puts "#{question_no}. #{question.msg}"
		file.puts "	N	%"
		answers = get_answer_nodes(xml_tree, question_no)
		if answers.size == 0
			file.write "No results for question #{question_no}.\n"
			return
		end
		choices = question.choices
		choices_desc = choices.map {|ch| ch.description}

		choices_count = answers[0].elements.size
		# go through all the possible choices
		# (partner, odrasel otrok, ...) 
		# and for each count the number of "yes"
		counts = (0..choices_count-1).map { |cur_choice|
			answers.inject(0) { |total, answer|
				choice = answer.elements[cur_choice+1]
				raise "can't parse xml #{val.name}" unless choice.name == "value"
				incr = choice.attributes['bool_no_bo'] == 'yes'
				if incr
					total + 1
				else
					total
				end
			}
		}
		counts.each_with_index { |count, i|
			file.puts "#{choices_desc[i]}	#{count}	#{percent(count, count_questionnaries(xml_tree))}"
		}
		file.puts "\n"
	end

	def export_order(xml_tree, questions, file, question_no,
			       must_all_choices=false)
		question = questions.find { |q| q.name == question_no}
		file.puts "#{question_no}. #{question.msg}"
		
		order_count = question.choices.select {|choic| choic.wants_order}.size
		if question.choices[0].order_max
			order_count = question.choices[0].order_max
		end
		answers = get_answer_nodes(xml_tree, question_no)

		selected_orders = []
		orders = answers.each { |answer|
			selected_orders_answer = []
			raise "xml parsing pb #{answer.name}" unless answer.name == 'fill_all_values'
			answer.each_element { |element|
				selected_order = element.attributes['order'].to_i
				selected_orders_answer.push(selected_order)
			}
			ok_to_add = true
			if must_all_choices
				# check all the choices were selected
				1.upto(order_count) { |order|
					if !selected_orders_answer.find { |a| a == order}
						ok_to_add = false
					end
				}
			end
			selected_orders.push(selected_orders_answer) if ok_to_add
		}
 
		# first line: put all the possible orders
		file.puts "\t" + (1..order_count).map {|i| i}.join("\t") + "\tSkupaj\tb.o.\tpovprečje ranga"

		question.choices.each_with_index { |choice, choice_index|
			orders_for_choice = selected_orders.inject([]) {|so_far, orders_4questionnary|
				so_far + [orders_4questionnary[choice_index]]
			
			}
			
			counts = (1..order_count).map { |order|
				orders_for_choice.select {|_order| _order == order}.size
			}
			# counts
			line = "#{choice.description}\t" + counts.join("\t")

			# add total
			total = orders_for_choice.inject(0) {|so_far, val|
				if val > 0
					so_far + 1
				else
					so_far
				end
			}
			line += "\t" + total.to_s
			# add b.o.
			line += "\t" + orders_for_choice.select {|v| v==0}.size.to_s
			# add povprečjega ranga
			if total == 0
				line += "\t0"
			else
				line += "\t" + "%.2f" % (orders_for_choice.inject(0) {|so_far,val| so_far + val} / total)
			end
			file.puts line
		}

		file.puts "\n"
	end
	
	# count every choice as a value of 1 and sum them up.
	def export_order_choice_count(xml_tree, questions, file, question_no)
		question = questions.find { |q| q.name == question_no}
		file.puts "#{question_no}. #{question.msg}"
		
		order_count = question.choices.select {|choic| choic.wants_order}.size
		if question.choices[0].order_max
			order_count = question.choices[0].order_max
		end
		answers = get_answer_nodes(xml_tree, question_no)

		selected_orders = []
		orders = answers.each { |answer|
			selected_orders_answer = []
			raise "xml parsing pb #{answer.name}" unless answer.name == 'fill_all_values'
			answer.each_element { |element|
				selected_order = element.attributes['order'].to_i
				selected_orders_answer.push(selected_order)
			}
			selected_orders.push(selected_orders_answer)
		}
		orders_counts = Array.new(selected_orders[0].size) {0}
		selected_orders[0].size.times { |index|
			selected_orders.each { |orders_4questionnary|
				orders_counts[index] += 1 if orders_4questionnary[index] > 0
			}
		}

		# first line: put all the possible orders
		file.puts "\tN\t%"

		question.choices.each_with_index { |choice, choice_index|
			file.puts "#{choice.description}\t#{orders_counts[choice_index]}"
		}

		file.puts "\n"
	end

	def export_value_table(xml_tree, questions, file,
			question_no, possible_values, answers_filter=nil)
		# first export the average of all
		# the number of old people helped (Q1) 
		question = questions.find { |q| q.name == question_no}
		file.puts "#{question_no}. #{question.msg}"
		file.puts "	N	%"
		elt_numbers = get_single_values_int(xml_tree, question_no,
			answers_filter)
		total = elt_numbers.size

		counts = possible_values.map { |possible_val|
			if possible_val =~ /(\d+)\+/
				min_val = $1.to_i
				elt_numbers.select {|val| !bo?(val) && val.to_i >= min_val }.size
			else
				cur_val = possible_val.to_i
				elt_numbers.select {|val| !bo?(val) && val.to_i == cur_val }.size
			end
		}
		count_bo = elt_numbers.select {|val| bo?(val)}.size
		possible_values.each_with_index { |possible_val, index|
			file.puts "#{possible_val}	#{counts[index]}	#{percent(counts[index], total)}"
		}
		file.puts "b.o.	#{count_bo}	#{percent(count_bo, total)}"
		file.puts "Skupaj	#{total}	"
		file.puts "\n"
	end

	def bo?(value)
		value == 'b.o.'
	end


	def get_single_values_int(xml_tree, question_no,
			answers_filter=nil)
		answers = get_answer_nodes(xml_tree, question_no).map { |answer_node|
			answer_node.attributes["value"]
		}
		if answers_filter
			answers = answers_filter.call(answers)
		end
		answers
	end

	def percent(count, total)
		"%.2f" % ((count * 100.0) / total)
	end

	def extension()
		"csv"
	end

	# here we create a new XML document, copying all
	# the questionnaries from the original that the user
	# selected for export. if the user selected nothing,
	# we return the original XML tree.
	def filter_xml_tree(_xml_tree, tv)
		result = REXML::Document.new
		result.add_element(REXML::Element.new("questionnaries"))
		is_selection = false
		tv.selection.selected_each { |tv, path, iter|
			is_selection = true
			questionnary_node = xml_for_questionnary_id(_xml_tree, iter[0])
			raise "problem with questionnary node #{questionnary_node.name}" unless questionnary_node.name == "questionnary"
			result.root.add_element(questionnary_node)
		}
		if is_selection
			result
		else
			_xml_tree
		end
	end

	def xml_for_questionnary_id(xml_tree, q_id)
		xml_tree.root.elements.find { |questionnary|
			raise "Invalid XML #{questionnary.name}" unless questionnary.name == 'questionnary'
			questionnary.attributes["number"] == q_id
		}
	end

end
