require 'plugins/export/shared/exportbase.rb'

class Spss < ExportBase
	def export_desc()
		"SPSS"
	end

	def extension()
		"csv"
	end

	# true if i support exporting only few questionnaries,
	# exporting only the selection.
	def support_export_selection()
		false
	end

	def export(_xml_tree, tv, questions, file, export_selected_if_possible)
		xml_tree = _xml_tree

		# we will export all the basic questions.
		# like a question with subquestions:
		# 
		# why do you do that?
		# - i feel the need for it [yes/no]
		# - there is no choice [yes/no]
		# - i feel satisfied doing it [yes/no]
		# the person can answer several of those subquestions.
		# for this export, we'll export a column by subquestion.
		question_descs = []
		header = questions.each { |q|
			if q.choices_type == :fill_all
				question_descs += q.choices.map {|c| q.name + "/" + c.description}
			else
				question_descs.push(q.name)
			end
		}
		header = question_descs.join "\t"
		file.puts header

		questionnary_nodes = xml_tree.root.elements

		# go through all the questionnaries
		questionnary_nodes.each { |questionnary_node|
			# export all the answers for this questionnary
			answers = []
			questions.each { |question|
				answers += formatted_answers(question, answer_node(questionnary_node, question.name))
			}
			file.puts answers.join("\t").gsub(/\r?\n/, ' ')
		}
	end

	# returns an array of answers
	# [6] or ["blue", 7]
	def formatted_answers(question, answer_node)
		raise answer_node.attributes['question'] unless answer_node.attributes['question'] == question.name

		case question.choices_type
		when :single_value
			raise answer_node.name unless answer_node.name == "single_val"
			[answer_node.attributes['value']]
		when :choice
			raise answer_node.name unless answer_node.name == 'choice_val'
			choices_desc = question.choices.map {|ch| ch.description}
			choice_index = answer_node.attributes['index'].to_i
			[choices_desc[choice_index]]
		when :fill_all
			raise answer_node.name unless answer_node.name == "fill_all_values"
			question.choices.zip(answer_node.elements.to_a).map { |choice, node|
				attr_name = nil
				if choice.wants_bool
					attr_name = 'bool'
				elsif choice.wants_bool_no_bo
					attr_name = 'bool_no_bo'
				elsif choice.wants_string
					attr_name = 'string'
				elsif choice.wants_int
					attr_name = 'int'
				elsif choice.wants_order
					attr_name = 'order'
				elsif choice.wants_text
					attr_name = 'text'
				elsif choice.alts
					attr_name = 'alts'
				elsif choice.values
					attr_name = 'values'
				end
				node.attributes[attr_name]
			}
		else
			raise "invalid question type #{question.choices_type}"
		end
	end
end
