module Studier;end
include Studier

class Studier::Choice

	attr_accessor :description
	attr_reader :wants_bool, :wants_bool_no_bo,
		:wants_string, :wants_text, :wants_int,
		:wants_order
	attr_reader :alts, :values
	attr_reader :order_max
	
	def initialize(choice)
		desc = choice
		@wants_string = reg_match(desc, /%s/)
		@wants_bool = reg_match(desc, /%b/)
		# bool is yes/no/no answer
		# bool_no_bo is yes/no strictly.
		@wants_bool_no_bo = reg_match(desc, /%B/)
		@wants_int = reg_match(desc, /%d/)
		@wants_text = reg_match(desc, /%text/)
		if desc =~ /%order(\d+)/
			@order_max = $1.to_i
		end
		@wants_order = reg_match(desc, /%order\d*/)

		if choice =~ /(.*)(\[.*\])$/
			desc = $1
			@alts = $2[1..-2].split(/\|/)
		elsif choice =~ /(.*)(\{.*\})$/
			desc = $1
			@values = $2[1..-2].split(/\|/)
		end
		@description = desc.strip
	end

	def reg_match(string, regex)
		result = (string =~ regex)
		string.sub!(regex, '') if result
		result
	end

	private :reg_match
end

class Studier::Question
	attr_accessor :msg

	# Array[Studier::Choice]
	attr_accessor :choices

	# question name. might
	# be different than
	# its index (eg 5a etc)
	attr_accessor :name
	
	# choices_type can be :choice, :fill_all, :single_value
	attr_accessor :choices_type

	# when some answers are chosen, you
	# jump to another question
	# gotos :: {question index => target question no}
	attr_accessor :gotos

	def initialize(msg, choices_type = nil, choices = nil)
		@msg = msg
		@choices_type = choices_type
		@choices = choices
		@gotos = {}
	end
end

def Studier::parse_questions(lines_enumerator)
	questions = []

	# question_lines :: [[string]]
	question_lines = []
	cur_line = nil
	lines_enumerator.each { |line|
		line.chomp!
		next if line =~ /(^#.*|^$)/

		if line =~ /^Q:(.*)/
			question_lines.push(cur_line) if cur_line
			cur_line = [$1]
		elsif line =~ /^(A|q):.*/
			cur_line.push(line)
		end
	}
	# push the last question
	question_lines.push(cur_line)

	question_lines.each_with_index { |question, index|
		msg, params = question.shift, question
		regexp = /\{(.+)\}/
		if msg =~ regexp
			question_name = $1
			msg.sub!(regexp, '')
		else
			question_name = (index +1).to_s
		end
		
		question = Question.new(msg)
		question.name = question_name

		goto_regexp = /\s*=>\s*(\d+)\s*$/
		jumps = params.each_with_index { |param, i|
		       next if param !~ goto_regexp
			# this question depends on
			# the bool result of another
			# question.
			goto = $1.to_i
			question.gotos[i] = goto
			param.sub!(goto_regexp, '')
		}
		case params[0]
		when 'A:%d'
			choices_type =:single_value
			choices = :int
		when 'A:%text'
			choices_type =:single_value
			choices = :text
		when 'A:%b'
			choices_type = :single_value
			choices = :bool
		when /^A:.*$/
			choices_type =:choice
			choices = params.map {|answer| answer[2..-1]}.map {|choice| Choice.new(choice)}
		when /^q:.*$/
			choices_type =:fill_all
			choices = params.map {|answer| answer[2..-1]}.map {|choice| Choice.new(choice)}
		else
			fail "unknown question params type: '#{params}'"
		end

		question.choices_type = choices_type
		question.choices = choices
		questions.push(question)
	}
	questions
end
