require 'test/unit'
include Test::Unit

require 'studier.rb'
include Studier

$QUESTIONS = <<EOF
Q:Koliko starim ljudjem hkrati nudite pomoč več kot 5 ur tedensko?
A:%d

Q:Koliko ljdujem, poleg stari osebi, še nudite podporo in oskrbo (npr. otrokom itd.)
A:%b

Q:{4a}V kakšnen sorodstvenem odnosu ste s staro osebo, ki jo oskrbujete?
A:sem zakonec/partner-ica
A:sem brat/sestra
A:sem otrok (hči/sin)
A:sem nečak/nečakinja
A:nisem v sorodstveni zvezi -sem njen/a %s
A:drugo %text

Q:Kaj je glavni razlog, da ste se odločili za oskrbovanje starega človeka?
q:Občutek dolžnosti %B
q:Stara oseba ni želela, da bi jo oskrboval kdo drug %b
q:Čustvena povezanost (ljubezen, hvaležnost) %B

Q:V Sloveniji so številni programi za pomoč in oskrbo starim ljudem. Katere so oblike pomoči, ki jih uporabljate/koristite?
q:dnevno varstvo [da|ne, ker te storitve ne poznam|ne, ker ne me zanima|ne, ker je storitev preveč draga|ne, ker storitev ni na voljo|ne, še ni potrebe|drugo|b.o.]
q:pomoč in nega na domu [da|ne, ker te storitve ne poznam|ne, ker ne me zanima|ne, ker je storitev preveč draga|ne, ker storitev ni na voljo|ne, še ni potrebe|drugo|b.o.]
q:"rdeči gumb" - telefon za pomoč na daljavo [da|ne, ker te storitve ne poznam|ne, ker ne me zanima|ne, ker je storitev preveč draga|ne, ker storitev ni na voljo|ne, še ni potrebe|drugo|b.o.]

Q:Prosimo vas, da napišite tisto, kar bi želeli še dodati, pa vas po tem nismo povpršali.
A:%text

Q:What do you prefer?
q:cats %order
q:dogs %order
q:kangaroos %order
EOF

$QUESTIONS_DEPS = <<EOF1
Q:do you like pets?
A:yes
A:no   => 2
A:b.o. => 2

Q:which ones?
A:cats
A:dogs
A:lions
A:penguins

Q:do you like people?
A:%b
EOF1

class TC_Studier < TestCase

	def test_parse_deps()
		questions = Studier::parse_questions($QUESTIONS_DEPS)
		assert(questions.size == 3)

		assert(questions[0].msg == 'do you like pets?')
		assert(questions[0].choices[0].description == "yes")
		assert(questions[0].choices[1].description == "no")
		assert(questions[0].choices[2].description == "b.o.")
		assert(questions[0].choices_type == :choice)
		assert(questions[0].gotos.size == 2)
		assert(questions[0].gotos[1] == 2)
		assert(questions[0].gotos[2] == 2)

		assert(questions[1].msg == 'which ones?')
		assert(questions[1].choices.size == 4)
		assert(questions[1].choices[0].description == "cats")
		assert(questions[1].choices[1].description == "dogs")
		assert(questions[1].choices[2].description == "lions")
		assert(questions[1].choices[3].description == "penguins")
		assert(questions[1].choices_type == :choice)

		assert(questions[2].msg == 'do you like people?')
		assert(questions[2].choices == :bool)
		assert(questions[2].choices_type == :single_value)
	end
	
	def test_parse()
		questions = Studier::parse_questions($QUESTIONS)

		assert(questions.size == 7)
		
		assert(questions[0].msg == 'Koliko starim ljudjem hkrati nudite pomoč več kot 5 ur tedensko?')
		assert(questions[0].choices == :int)
		assert(questions[0].choices_type == :single_value)

		assert(questions[1].msg == 'Koliko ljdujem, poleg stari osebi, še nudite podporo in oskrbo (npr. otrokom itd.)')
		assert(questions[1].choices == :bool)
		assert(questions[1].choices_type == :single_value)
		assert(questions[1].name == '2')

		assert(questions[2].msg == 'V kakšnen sorodstvenem odnosu ste s staro osebo, ki jo oskrbujete?')
		assert(questions[2].choices[0].description == 'sem zakonec/partner-ica')
		assert(questions[2].choices[1].description == 'sem brat/sestra')
		assert(questions[2].choices[2].description == 'sem otrok (hči/sin)')
		assert(questions[2].choices[3].description == 'sem nečak/nečakinja')
		assert(!questions[2].choices[3].wants_string)
		assert(!questions[2].choices[3].wants_bool)
		assert(questions[2].choices[4].description == 'nisem v sorodstveni zvezi -sem njen/a')
		assert(questions[2].choices[4].wants_string)
		assert(questions[2].choices[4].description == 'nisem v sorodstveni zvezi -sem njen/a')
		assert(questions[2].choices[5].description == 'drugo')
		assert(questions[2].choices[5].wants_text)
		assert(questions[2].choices_type == :choice)
		assert(questions[2].name == '4a')
		assert(questions[2].choices[5].wants_text)

		assert(questions[3].msg == 'Kaj je glavni razlog, da ste se odločili za oskrbovanje starega človeka?')
		assert(questions[3].choices[0].description == 'Občutek dolžnosti')
		assert(questions[3].choices[0].wants_bool_no_bo)
		assert(questions[3].choices[1].description == 'Stara oseba ni želela, da bi jo oskrboval kdo drug')
		assert(questions[3].choices[1].wants_bool)
		assert(questions[3].choices[2].description == 'Čustvena povezanost (ljubezen, hvaležnost)')
		assert(questions[3].choices[2].wants_bool_no_bo)
		assert(questions[3].choices_type == :fill_all)

		assert(questions[4].msg == 'V Sloveniji so številni programi za pomoč in oskrbo starim ljudem. Katere so oblike pomoči, ki jih uporabljate/koristite?')
		assert(questions[4].choices[0].description == 'dnevno varstvo')
		assert(questions[4].choices[0].alts == ['da', 'ne, ker te storitve ne poznam', 'ne, ker ne me zanima', 'ne, ker je storitev preveč draga', 'ne, ker storitev ni na voljo', 'ne, še ni potrebe', 'drugo', 'b.o.'])
		assert(questions[4].choices[1].description == 'pomoč in nega na domu')
		assert(questions[4].choices[1].alts == ['da', 'ne, ker te storitve ne poznam', 'ne, ker ne me zanima', 'ne, ker je storitev preveč draga', 'ne, ker storitev ni na voljo', 'ne, še ni potrebe', 'drugo', 'b.o.'])
		assert(questions[4].choices[2].description == '"rdeči gumb" - telefon za pomoč na daljavo')
		assert(questions[4].choices[2].alts == ['da', 'ne, ker te storitve ne poznam', 'ne, ker ne me zanima', 'ne, ker je storitev preveč draga', 'ne, ker storitev ni na voljo', 'ne, še ni potrebe', 'drugo', 'b.o.'])
	       assert(questions[4].choices_type == :fill_all)

		assert(questions[5].msg == 'Prosimo vas, da napišite tisto, kar bi želeli še dodati, pa vas po tem nismo povpršali.')
		assert(questions[5].choices == :text)
		assert(questions[5].choices_type == :single_value)

		assert(questions[6].msg == 'What do you prefer?')
		assert(questions[6].choices[0].description == 'cats')
		assert(questions[6].choices[0].wants_order)
		assert(!questions[6].choices[0].wants_string)
		assert(!questions[6].choices[0].wants_text)
		assert(questions[6].choices[1].description == 'dogs')
		assert(questions[6].choices[1].wants_order)
		assert(questions[6].choices[2].description == 'kangaroos')
		assert(questions[6].choices[2].wants_order)
		assert(questions[6].choices_type == :fill_all)
	end
end

require 'test/unit/ui/console/testrunner'
UI::Console::TestRunner.run(TC_Studier)
